@@ -28,30 +28,251 @@ from xalpha import __path__
 logger = logging.getLogger(__name__)
 
 # date obj of today
-today = lambda: dt.datetime.combine(dt.date.today(), dt.time.min)
+# today = lambda: dt.datetime.combine(dt.date.today(), dt.time.min)
 
-# string for yesterday, only used for indexinfo url
-yesterday = lambda: dt.datetime.strftime(
-    (dt.datetime.now() - dt.timedelta(1)), "%Y%m%d"
-)
+tz_bj = dt.timezone(dt.timedelta(hours=8))
+
+
+def today_obj():
+    """
+    today obj in beijing timezone with no tzinfo
+
+    :return: datetime.datetime
+    """
+    now = dt.datetime.now(tz=tz_bj)
+    return now.replace(hour=0, minute=0, second=0, microsecond=0).replace(tzinfo=None)
 
-# string for yesterday with dash
-yesterdaydash = lambda: dt.datetime.strftime(
-    (dt.datetime.now() - dt.timedelta(1)), "%Y-%m-%d"
-)
 
 # datetime obj for yesterdate date with time set to be 0:0:0
-yesterdayobj = lambda: dt.datetime.strptime(yesterdaydash(), "%Y-%m-%d")
+yesterdayobj = lambda: (dt.datetime.now(tz_bj).replace(tzinfo=None) - dt.timedelta(1))
+
+# string for yesterday, only used for indexinfo url
+yesterday = lambda: dt.datetime.strftime(yesterdayobj(), "%Y%m%d")
+
+# string for yesterday with dash
+yesterdaydash = lambda: dt.datetime.strftime(yesterdayobj(), "%Y-%m-%d")
+
 
 # list: all the trade date of domestic stock market in the form of string
 caldate = pd.read_csv(os.path.join(__path__[0], "caldate.csv"))
 opendate = list(caldate[caldate["is_open"] == 1]["cal_date"])
-# directly use the tushare API instead of import tushare package for simplicity
 # opendate = list(ts.trade_cal()[ts.trade_cal()['isOpen']==1]['calendarDate'])
 
 # fund code list which always round down for the purchase share approximation
 droplist = ["003318", "000311"]
 
+region_trans = {
+    "瑞士": "CH",
+    "日本": "JP",
+    "韩国": "KR",
+    "美国": "US",
+    "香港": "HK",
+    "中国香港": "HK",
+    "德国": "DE",
+    "英国": "UK",
+    "法国": "FR",
+    "中国": "CN",
+    "墨西哥": "MX",
+    "澳大利亚": "AU",
+    "新加坡": "SG",
+    "印度": "IN",
+    "台湾": "TW",
+    "中国台湾": "TW",
+}
+
+# extract from xa.misc.get_tdx_holidays
+holidays = {
+    "AU": [
+        "2020-01-01",
+        "2020-01-27",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-04-25",
+        "2020-06-08",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-28",
+        "2020-12-31",
+    ],
+    "CH": [
+        "2020-01-01",
+        "2020-01-02",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-05-21",
+        "2020-06-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+    ],
+    "CN": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-01-29",
+        "2020-01-30",
+        "2020-01-31",
+        "2020-04-06",
+        "2020-05-01",
+        "2020-05-04",
+        "2020-05-05",
+        "2020-06-25",
+        "2020-06-26",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-05",
+        "2020-10-06",
+        "2020-10-07",
+        "2020-10-08",
+    ],
+    "DE": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-06-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+    ],
+    "FR": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-01",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+    ],
+    "HK": [
+        "2020-01-01",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-04-30",
+        "2020-05-01",
+        "2020-06-25",
+        "2020-07-01",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-26",
+        "2020-12-25",
+    ],
+    "IN": [
+        "2020-02-21",
+        "2020-03-10",
+        "2020-04-02",
+        "2020-04-06",
+        "2020-04-10",
+        "2020-04-14",
+        "2020-05-01",
+        "2020-05-25",
+        "2020-10-02",
+        "2020-11-16",
+        "2020-11-30",
+        "2020-12-25",
+    ],
+    "JP": [
+        "2020-01-01",
+        "2020-01-02",
+        "2020-01-03",
+        "2020-01-13",
+        "2020-02-11",
+        "2020-02-24",
+        "2020-03-20",
+        "2020-04-29",
+        "2020-05-04",
+        "2020-05-05",
+        "2020-05-06",
+        "2020-07-23",
+        "2020-07-24",
+        "2020-08-10",
+        "2020-09-21",
+        "2020-09-22",
+        "2020-11-03",
+        "2020-11-23",
+        "2020-12-31",
+    ],
+    "KR": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-04-30",
+        "2020-05-01",
+        "2020-05-05",
+        "2020-09-30",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-09",
+        "2020-12-25",
+        "2020-12-31",
+    ],
+    "SG": [
+        "2020-01-01",
+        "2020-01-24",
+        "2020-04-10",
+        "2020-05-01",
+        "2020-05-07",
+        "2020-05-21",
+        "2020-07-31",
+        "2020-08-10",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-31",
+    ],
+    "TW": [
+        "2020-01-01",
+        "2020-01-21",
+        "2020-01-22",
+        "2020-01-23",
+        "2020-01-24",
+        "2020-01-27",
+        "2020-01-28",
+        "2020-01-29",
+        "2020-02-28",
+        "2020-04-02",
+        "2020-04-03",
+        "2020-05-01",
+        "2020-06-25",
+        "2020-06-26",
+        "2020-10-01",
+        "2020-10-02",
+        "2020-10-09",
+    ],
+    "UK": [
+        "2020-01-01",
+        "2020-04-10",
+        "2020-04-13",
+        "2020-05-08",
+        "2020-05-25",
+        "2020-08-31",
+        "2020-12-24",
+        "2020-12-25",
+        "2020-12-28",
+        "2020-12-31",
+        "2021-01-01",
+    ],
+    "US": [
+        "2020-01-01",
+        "2020-01-20",
+        "2020-02-17",
+        "2020-03-08",
+        "2020-04-10",
+        "2020-05-25",
+        "2020-07-03",
+        "2020-09-07",
+        "2020-11-01",
+        "2020-11-26",
+        "2020-11-27",
+        "2020-12-24",
+        "2020-12-25",
+        "2021-01-01",
+    ],
+}
+
 connection_errors = (
     ConnectionResetError,
     requests.exceptions.RequestException,
@@ -82,10 +303,10 @@ heatmap_opts = {
     )
 }
 
-pie_opts = {
-    "tooltip_opts": TooltipOpts(),
-    "legend_opts": LegendOpts(orient="vertical", pos_left="left"),
-}
+# pie_opts = {
+#     "tooltip_opts": TooltipOpts(),
+#     "legend_opts": LegendOpts(orient="vertical", pos_left="left"),
+# }
 
 themeriver_opts = {
     "xaxis_opts": AxisOpts(type_="time"),
@@ -158,7 +379,19 @@ def convert_date(date):
         return date
 
 
+def _date_check(dtobj, check=False):
+    if not isinstance(dtobj, dt.datetime):
+        dtobj = dt.datetime.strptime(dtobj.replace("/", "").replace("-", ""), "%Y%m%d")
+    if check and (dtobj.year > 2020 or dtobj.year < 1991):
+        # TODO: remember change 2020 every year!
+        raise ValueError(
+            "date goes beyond market range: %s" % dtobj.strftime("%Y-%m-%d")
+        )
+    return dtobj
+
+
 def next_onday(dtobj):
+    dtobj = _date_check(dtobj, check=True)
     dtobj += dt.timedelta(1)
     while dtobj.strftime("%Y-%m-%d") not in opendate:
         dtobj += dt.timedelta(1)
@@ -166,6 +399,7 @@ def next_onday(dtobj):
 
 
 def last_onday(dtobj):
+    dtobj = _date_check(dtobj, check=True)
     dtobj -= dt.timedelta(1)
     while dtobj.strftime("%Y-%m-%d") not in opendate:
         dtobj -= dt.timedelta(1)
@@ -185,6 +419,22 @@ def scale_dict(d, scale=1, ulimit=100, dlimit=50, aim=None):
     return d
 
 
+def _float(n):
+    try:
+        n = n.replace(",", "")
+        if n.endswith("K") or n.endswith("k"):
+            n = float(n[:-1]) * 1000
+        elif n.endswith("M") or n.endswith("m"):
+            n = float(n[:-1]) * 1000 * 1000
+        elif n.endswith("G") or n.endswith("g") or n.endswith("B") or n.endswith("b"):
+            n = float(n[:-1]) * 1000 * 1000 * 1000
+        elif n == "-":
+            return 0
+    except AttributeError:
+        pass
+    return float(n)
+
+
 def reconnect(tries=5, timeout=12):
     def robustify(f):
         @wraps(f)
@@ -199,7 +449,12 @@ def reconnect(tries=5, timeout=12):
                 url = args[0]
             else:
                 url = kws.get("url", "")
-
+            headers = kws.get("headers", {})
+            if (not headers.get("user-agent", None)) and (
+                not headers.get("User-Agent", None)
+            ):
+                headers["user-agent"] = "Mozilla/5.0"
+            kws["headers"] = headers
             for count in range(tries):
                 try:
                     logger.debug(
@@ -211,7 +466,7 @@ def reconnect(tries=5, timeout=12):
                 except connection_errors as e:
                     logger.warning("Fails at fetching url: %s. Try again." % url)
                     if count == tries - 1:
-                        logger.critical(
+                        logger.error(
                             "Still wrong at fetching url: %s. after %s tries."
                             % (url, tries)
                         )
@@ -311,6 +566,6 @@ holdings["501018"] = {
     "etfs/united-states-oil-fund": 9.63,
 }
 holdings["501018rt"] = {
-    "commodities/brent-oil": 40 * 0.9,
-    "commodities/crude-oil": 60 * 0.9,
+    "commodities/brent-oil": {"weight": 49, "time": -1},
+    "commodities/crude-oil": {"weight": 45, "time": 4},
 }
