@@ -4,7 +4,9 @@ module for implementation of indicator class, which is designed as MinIn for sys
 """
 
 import pandas as pd
-from pyecharts.charts import Line
+from pyecharts import options as opts
+from pyecharts.charts import Kline, Line, Bar, Grid
+from pyecharts.commons.utils import JsCode
 
 from xalpha.cons import line_opts, opendate, yesterdayobj
 
@@ -64,7 +66,7 @@ class indicator:
         """
         generate price table for mulfix class, the cinfo class has this attr by default
         """
-        if getattr(self, "price", None) is None:
+        if getattr(self, "price", None) is None:  # 基金组合类，而非基金信息类
             times = pd.date_range(self.totcftable.iloc[0].date, yesterdayobj())
             netvalue = []
             for date in times:
@@ -96,6 +98,7 @@ class indicator:
             4,
         )
 
+    @staticmethod
     def annualized_returns(price, start, date=yesterdayobj()):
         """
         :param price: price table of info().price
@@ -115,6 +118,34 @@ class indicator:
     def benchmark_annualized_returns(self, date=yesterdayobj()):
         return indicator.annualized_returns(self.bmprice, self.start, date)
 
+    def pct_chg(self, freq="Y", benchmark=True):
+        """
+        年度，月，周涨幅统计
+
+        :param freq: str, default Y, could be M or W or anything pd.date_range accepts
+        :return: pd.DataFrame with columns date and pct_chg
+        """
+        if getattr(self, "bmprice", None) is None:
+            benchmark = False
+
+        ydf = pd.merge_asof(
+            pd.DataFrame(
+                pd.date_range(
+                    self.price["date"].iloc[0], self.price["date"].iloc[-1], freq=freq
+                ),
+                columns=["date"],
+            ),
+            self.price,
+        )
+        ydf["pct_chg"] = ydf["netvalue"].pct_change()
+        if benchmark:
+            ydf = pd.merge_asof(ydf, self.bmprice, on="date", suffixes=["", "_bc"])
+            ydf["pct_chg_benchmark"] = ydf["netvalue_bc"].pct_change()
+            ydf["pct_chg_difference"] = ydf["pct_chg"] - ydf["pct_chg_benchmark"]
+            return ydf[["date", "pct_chg", "pct_chg_benchmark", "pct_chg_difference"]]
+
+        return ydf[["date", "pct_chg"]]
+
     def beta(self, date=yesterdayobj()):
         bcmk = indicator.ratedaily(self.bmprice, date)
         bt = indicator.ratedaily(self.price, date)
@@ -143,13 +174,15 @@ class indicator:
             (res.loc["bcmk", "bcmk"] ** 0.5) * res.loc["bt", "bt"] ** 0.5
         )
 
+    @staticmethod
     def ratedaily(price, date=yesterdayobj()):
         partp = price[price["date"] <= date]
-        return [
-            (partp.iloc[i + 1].netvalue - partp.iloc[i].netvalue)
-            / partp.iloc[i].netvalue
-            for i in range(len(partp) - 1)
-        ]
+        return list(partp["netvalue"].pct_change())[1:]
+        # return [
+        #     (partp.iloc[i + 1].netvalue - partp.iloc[i].netvalue)
+        #     / partp.iloc[i].netvalue
+        #     for i in range(len(partp) - 1)
+        # ]
 
     def volatility(price, date=yesterdayobj()):
         df = pd.DataFrame(data={"rate": indicator.ratedaily(price, date)})
@@ -427,7 +460,7 @@ class indicator:
 
     ## 以下是可视化部分
 
-    def v_netvalue(self, end=yesterdayobj(), benchmark=True, vopts=None):
+    def v_netvalue(self, end=yesterdayobj(), benchmark=True, rendered=True, vopts=None):
         """
         visulaization on  netvalue curve
 
@@ -435,7 +468,12 @@ class indicator:
         :param benchmark: bool, whether include benchmark's netvalue curve, default true
         :param vopts: dict, options for pyecharts instead of builtin settings
         """
-        a, b = self.comparison(end)
+        if getattr(self, "bmprice", None) is None:
+            benchmark = False
+        if benchmark:
+            a, b = self.comparison(end)
+        else:
+            a = self.price
         if vopts is None:
             vopts = line_opts
         line = Line()
@@ -444,15 +482,18 @@ class indicator:
             y_axis=list(a.netvalue), series_name=self.name, is_symbol_show=False
         )
         line.set_global_opts(**vopts)
-        if benchmark is True:
+        if benchmark:
             line.add_yaxis(
                 series_name=self.benchmark.name,
                 y_axis=list(b.netvalue),
                 is_symbol_show=False,
             )
-        return line.render_notebook()
+        if rendered:
+            return line.render_notebook()
+        else:
+            return line
 
-    def v_techindex(self, end=yesterdayobj(), col=None, vopts=None):
+    def v_techindex(self, end=yesterdayobj(), col=None, rendered=True, vopts=None):
         """
         visualization on netvalue curve and specified indicators
 
@@ -475,4 +516,140 @@ class indicator:
             for ind in col:
                 inddata = list(partprice[ind])
                 line.add_yaxis(series_name=ind, y_axis=inddata, is_symbol_show=False)
-        return line.render_notebook()
+        if rendered:
+            return line.render_notebook()
+        else:
+            return line
+
+
+def plot_kline(df, rendered=True, col=""):
+    """
+    针对 dataframe 直接画出标准看盘软件的上k线图下成交量图的形式
+
+    :param df:
+    :param rendered:
+    :param col:
+    :return:
+    """
+    kline = (
+        Kline()
+        .add_xaxis(xaxis_data=list(df["date"]))
+        .add_yaxis(
+            series_name="",
+            itemstyle_opts=opts.ItemStyleOpts(
+                color="#ef232a",
+                color0="#14b143",
+                border_color="#ef232a",
+                border_color0="#14b143",
+            ),
+            y_axis=list(zip(df["open"], df["close"], df["high"], df["low"])),
+            markpoint_opts=opts.MarkPointOpts(
+                data=[
+                    opts.MarkPointItem(type_="max", name="最大值"),
+                    opts.MarkPointItem(type_="min", name="最小值"),
+                ],
+                symbol="pin",
+                symbol_size=[56, 40],
+                # label_opts=opts.LabelOpts(color="#CCFFFF",position=["top", "bottom"])
+            ),
+        )
+        .set_global_opts(
+            datazoom_opts=[
+                opts.DataZoomOpts(
+                    is_show=True,
+                    type_="slider",
+                    range_start=50,
+                    range_end=100,
+                    xaxis_index=[0, 1],
+                ),
+                opts.DataZoomOpts(
+                    is_show=False,
+                    type_="inside",
+                    range_start=50,
+                    range_end=100,
+                    xaxis_index=1,
+                ),
+            ],
+            tooltip_opts=opts.TooltipOpts(
+                is_show=True,
+                trigger="axis",
+                trigger_on="mousemove",
+                axis_pointer_type="cross",
+            ),
+        )
+    )
+    if col:
+        line = (
+            Line()
+            .add_xaxis(xaxis_data=list(df["date"]))
+            .add_yaxis(
+                series_name="",
+                y_axis=list(df[col]),
+                is_smooth=True,
+                linestyle_opts=opts.LineStyleOpts(opacity=0.5),
+                label_opts=opts.LabelOpts(is_show=False),
+            )
+        )
+        kline = kline.overlap(line)
+
+    if "volume" in df.columns:
+        vl = list(df["volume"])
+    elif "amount" in df.columns:
+        vl = list(df["amount"])
+    else:
+        vl = [0 for _ in range(len(df))]
+    bar = (
+        Bar()
+        .add_js_funcs("var barData = {}".format(list(df["close"] - df["open"])))
+        .add_xaxis(xaxis_data=list(df["date"]))
+        .add_yaxis(
+            series_name="",
+            yaxis_data=vl,
+            label_opts=opts.LabelOpts(is_show=False),
+            itemstyle_opts=opts.ItemStyleOpts(
+                color=JsCode(
+                    """
+                function(params) {
+                    var colorList;
+                    if (barData[params.dataIndex]>0) {
+                        colorList = '#ef232a';
+                    } else {
+                        colorList = '#14b143';
+                    }
+                    return colorList;
+                }
+                """
+                )
+            ),
+        )
+        .set_global_opts(
+            tooltip_opts=opts.TooltipOpts(
+                is_show=True,
+                trigger="axis",
+                trigger_on="mousemove",
+                axis_pointer_type="cross",
+            ),
+        )
+    )
+    grid_chart = Grid()
+    grid_chart.add_js_funcs("var barData = {}".format(list(df["close"] - df["open"])))
+    grid_chart.add(
+        kline,
+        grid_opts=opts.GridOpts(
+            pos_left="10%", pos_right="1%", pos_top="2%", height="65%"
+        ),
+    )
+
+    grid_chart.add(
+        bar,
+        grid_opts=opts.GridOpts(
+            pos_left="10%", pos_right="1%", pos_top="71%", height="22%"
+        ),
+    )
+    if rendered:
+        return grid_chart.render_notebook()
+    else:
+        return grid_chart
+
+
+pd.DataFrame.v_kline = plot_kline
