@@ -2,13 +2,16 @@
 """
 modules of info class, including cashinfo, indexinfo and fundinfo class
 """
+
+import os
 import csv
 import datetime as dt
 import json
 import re
+import logging
+from functools import lru_cache
 
 import pandas as pd
-import requests as rq
 from bs4 import BeautifulSoup
 from sqlalchemy import exc
 
@@ -21,14 +24,15 @@ from xalpha.cons import (
     yesterday,
     yesterdaydash,
     yesterdayobj,
-    today,
-    connection_errors,
+    today_obj,
     rget,
+    _float,
 )
-from xalpha.exceptions import FundTypeError, TradeBehaviorError
+from xalpha.exceptions import FundTypeError, TradeBehaviorError, ParserFailure
 from xalpha.indicator import indicator
 
 _warnmess = "Something weird on redem fee, please adjust self.segment by hand"
+logger = logging.getLogger(__name__)
 
 
 def _shengoucal(sg, sgf, value, label):
@@ -69,8 +73,10 @@ def _nfloat(string):
                 result = -float(re.match(r".*折算(\d*\.\d*)\D*", string).group(1))
             elif re.match(r'"拆分\D*(\d*\.\d*)\D*"', string):
                 result = -float(re.match(r'"拆分\D*(\d*\.\d*)\D*"', string).group(1))
+            elif re.match(r"\D*分拆(\d*\.\d*)\D*", string):
+                result = -float(re.match(r"\D*分拆(\d*\.\d*)\D*", string).group(1))
             else:
-                print("The comment col cannot be converted: %s" % string)
+                logger.warning("The comment col cannot be converted: %s" % string)
                 result = string
     return result
 
@@ -185,6 +191,101 @@ class FundReport:
         return d
 
 
+@lru_cache()
+def get_fund_holdings(code, year="", season="", month="", category="jjcc"):
+    """
+    获取基金详细的底层持仓信息
+
+    :param code: str. 6 位基金代码
+    :param year:  int. eg. 2019
+    :param season: int, 1,2,3,4
+    :param month: Optional[int]. 指定 season 即可，一般不需理会
+    :param category: str. stock 股票持仓， bond 债券持仓，天天基金无法自动处理海外基金持仓，暂未兼容 FOF 的国内基金持仓
+    :return: pd.DataFrame or None. 没有对应持仓时返回 None。
+    """
+    if not month and season:
+        month = 3 * int(season)
+    if category in ["stock", "stocks", "jjcc", "", "gp", "s"]:
+        category = "jjcc"
+    elif category in ["bond", "bonds", "zq", "zqcc", "b"]:
+        category = "zqcc"
+    else:
+        raise ParserFailure("unrecognized category %s" % category)
+    if code.startswith("F"):
+        code = code[1:]
+    r = rget(
+        "http://fundf10.eastmoney.com/FundArchivesDatas.aspx?type={category}&code={code}&topline=10&\
+year={year}&month={month}".format(
+            year=str(year), month=str(month), code=code, category=category
+        ),
+        headers={
+            "Host": "fundf10.eastmoney.com",
+            "Referer": "http://fundf10.eastmoney.com/ccmx_{code}.html".format(
+                code=code
+            ),
+        },
+    )
+    if len(r.text) < 50:
+        return
+        # raise ParserFailure(
+        #     "This fund has no holdings on stock or bonds in this period"
+        # )
+    s = BeautifulSoup(
+        re.match("[\s\S]*apidata={ content:(.*),arryear:", r.text).groups()[0], "lxml"
+    )
+    if len(s.text) < 30:
+        return
+        # raise ParserFailure(
+        #     "This fund has no holdings on stock or bonds in this period"
+        # )
+    timeline = [
+        i.string for i in s.findAll("font", class_="px12") if i.text.startswith("2")
+    ]
+    ind = 0
+    if month:
+        for i, d in enumerate(timeline):
+            if d.split("-")[1][-1] == str(month)[-1]:  # avoid 09 compare to 9
+                ind = i
+                break
+        else:
+            return  # not update to this month
+    t1 = s.findAll("table")[ind]
+    main = [[j.text for j in i.contents] for i in t1.findAll("tr")[1:]]
+    cols = [j.text for j in t1.findAll("tr")[0].contents if j.text.strip()]
+    icode = 1
+    iname = 2
+    iratio = 4
+    ishare = 5
+    ivalue = 6
+    for j, col in enumerate(cols):
+        if col.endswith("代码"):
+            icode = j
+        elif col.endswith("名称"):
+            iname = j
+        elif col.endswith("比例"):
+            iratio = j
+        elif col.startswith("持股数"):
+            ishare = j
+        elif col.startswith("持仓市值"):
+            ivalue = j
+    if category == "jjcc":
+        result = {"code": [], "name": [], "ratio": [], "share": [], "value": []}
+        for l in main:
+            result["code"].append(l[icode])
+            result["name"].append(l[iname])
+            result["ratio"].append(float(l[iratio][:-1]))
+            result["share"].append(_float(l[ishare]))
+            result["value"].append(_float(l[ivalue]))
+    elif category == "zqcc":
+        result = {"code": [], "name": [], "ratio": [], "value": []}
+        for l in main:
+            result["code"].append(l[1])
+            result["name"].append(l[2])
+            result["ratio"].append(float(l[3][:-1]))
+            result["value"].append(_float(l[4]))
+    return pd.DataFrame(result)
+
+
 class basicinfo(indicator):
     """
     Base class for info of fund, index or even cash,
@@ -214,13 +315,24 @@ class basicinfo(indicator):
     ):
         # 增量 IO 的逻辑都由 basicinfo 类来处理，对于具体的子类，只需实现_save_form 和 _fetch_form 以及 update 函数即可
         self.code = code
-        self.format = form
+
         self.round_label = round_label
         self.dividend_label = dividend_label
         self.value_label = value_label
         self.specialdate = []
         self.fenhongdate = []
         self.zhesuandate = []
+
+        # compatible with new xa.set_backend()
+        import xalpha.universal as xu
+
+        if (xu.ioconf["backend"] in ["csv", "sql"]) and (not path):
+            fetch = True
+            save = True
+            form = xu.ioconf["backend"]
+            path = xu.ioconf["path"]
+            path = os.path.join(path, xu.ioconf["prefix"] + "INFO-")
+        self.format = form
         if fetch is False:
             self._basic_init()  # update self. name rate and price table
         else:
@@ -231,7 +343,7 @@ class basicinfo(indicator):
                     self.save(path, self.format, option="a", delta=df)
 
             except (FileNotFoundError, exc.ProgrammingError) as e:
-                print("no saved copy of %s" % self.code)
+                logger.info("no saved copy of %s" % self.code)
                 fetch = False
                 self._basic_init()
 
@@ -402,17 +514,25 @@ class fundinfo(basicinfo):
         save=False,
         path="",
         form="csv",
+        priceonly=False,
     ):
         if round_label == 1 or (code in droplist):
             label = 1  # the scheme of round down on share purchase
         else:
             label = 0
+        if code.startswith("F") and code[1:].isdigit():
+            code = code[1:]
+        elif code.startswith("M") and code[1:].isdigit():
+            raise FundTypeError(
+                "This code seems to be a mfund, use ``mfundinfo`` instead"
+            )
         self._url = (
             "http://fund.eastmoney.com/pingzhongdata/" + code + ".js"
         )  # js url api for info of certain fund
         self._feeurl = (
             "http://fund.eastmoney.com/f10/jjfl_" + code + ".html"
         )  # html url for trade fees info of certain fund
+        self.priceonly = priceonly
 
         super().__init__(
             code,
@@ -435,15 +555,21 @@ class fundinfo(basicinfo):
 
     def _basic_init(self):
         self._page = rget(self._url)
+        if self._page.status_code == 404:
+            raise ParserFailure("Unrecognized fund, please check fund code you input.")
         if self._page.text[:800].find("Data_millionCopiesIncome") >= 0:
             raise FundTypeError("This code seems to be a mfund, use mfundinfo instead")
 
-        l = re.match(r".*Data_netWorthTrend = ([^;]*);.*", self._page.text).groups()[0]
+        l = re.match(
+            r"[\s\S]*Data_netWorthTrend = ([^;]*);[\s\S]*", self._page.text
+        ).groups()[0]
         l = l.replace("null", "None")  # 暂未发现基金净值有 null 的基金，若有，其他地方也很可能出问题！
         l = eval(l)
-        ltot = re.match(r".*Data_ACWorthTrend = ([^;]*);.*", self._page.text).groups()[
+        ltot = re.match(
+            r"[\s\S]*Data_ACWorthTrend = ([^;]*);[\s\S]*", self._page.text
+        ).groups()[
             0
-        ]
+        ]  # .* doesn't match \n
         ltot = ltot.replace("null", "None")  ## 096001 总值数据中有 null！
         ltot = eval(ltot)
         ## timestamp transform tzinfo must be taken into consideration
@@ -464,13 +590,19 @@ class fundinfo(basicinfo):
 
         try:
             rate = float(
-                eval(re.match(r".*fund_Rate=([^;]*);.*", self._page.text).groups()[0])
+                eval(
+                    re.match(
+                        r"[\s\S]*fund_Rate=([^;]*);[\s\S]*", self._page.text
+                    ).groups()[0]
+                )
             )
         except ValueError:
             rate = 0
-            print("warning: this fund has no data for rate")  # know cases: 510030
+            logger.info("warning: this fund has no data for rate")  # know cases: ETF
 
-        name = eval(re.match(r".*fS_name = ([^;]*);.*", self._page.text).groups()[0])
+        name = eval(
+            re.match(r"[\s\S]*fS_name = ([^;]*);[\s\S]*", self._page.text).groups()[0]
+        )
 
         self.rate = rate
         # shengou rate in tiantianjijin, daeshengou rate discount is not considered
@@ -478,9 +610,12 @@ class fundinfo(basicinfo):
         df = pd.DataFrame(data=infodict)
         df = df[df["date"].isin(opendate)]
         df = df.reset_index(drop=True)
+        if len(df) == 0:
+            raise ParserFailure("no price table found for this fund %s" % self.code)
         self.price = df[df["date"] <= yesterdaydash()]
         # deal with the redemption fee attrs finally
-        self._feepreprocess()
+        if not self.priceonly:
+            self._feepreprocess()
 
     def _feepreprocess(self):
         """
@@ -490,41 +625,84 @@ class fundinfo(basicinfo):
         soup = BeautifulSoup(
             feepage.text, "lxml"
         )  # parse the redemption fee html page with beautiful soup
-        self.feeinfo = [
-            item.string
-            for item in soup.findAll("a", {"name": "shfl"})[
-                0
-            ].parent.parent.next_sibling.next_sibling.find_all("td")
-            if item.string != "---"
-        ]
+        somethingwrong = False
+        if not soup.findAll("a", {"name": "shfl"}):
+            somethingwrong = True
+            logger.warning("%s 基金赎回信息为空，可能由于该基金已终止运作" % self.code)
+            self.feeinfo = []
+        else:
+            self.feeinfo = [
+                item.string
+                for item in soup.findAll("a", {"name": "shfl"})[
+                    0
+                ].parent.parent.next_sibling.next_sibling.find_all("td")
+                if item.string != "---"
+            ]
         # this could be [], known case 510030
-        if not self.feeinfo:
+
+        if not self.feeinfo or len(self.feeinfo) % 2 != 0:
+            somethingwrong = True
+        else:
+            for item in self.feeinfo:
+                if "开放期" in item or "封闭" in item or "开放日期" in item or "运作期" in item:
+                    # 暂时没有完美维护定开基金赎回费处理的计划
+                    somethingwrong = True
+        if somethingwrong:
+            logger.warning(
+                "%s 赎回费信息异常，多是因为定开基金，封闭基金或场内 ETF: %s" % (self.code, self.feeinfo)
+            )
             self.feeinfo = ["小于7天", "1.50%", "大于等于7天", "0.00%"]
-        self.segment = fundinfo._piecewise(self.feeinfo)
+        # print(self.feeinfo)
+        try:
+            self.segment = fundinfo._piecewise(self.feeinfo)
+        except (ValueError, IndexError) as e:
+            logger.warning(
+                "%s 赎回费信息抓取异常，请手动设定 ``self.segment`` 和 ``self.feeinfo``: %s"
+                % (self.code, self.feeinfo)
+            )
+            # below is default one
+            self.feeinfo = ["小于7天", "1.50%", "大于等于7天", "0.00%"]
+            self.segment = fundinfo._piecewise(self.feeinfo)
 
     @staticmethod
     def _piecewise(a):
         """
         Transform the words list into a pure number segment list for redemption fee, eg. [[0,7],[7,365],[365]]
         """
+
         b = [
             (
                 a[2 * i]
+                .replace("持有期限", "")
+                .replace("开放运作期时持有", "")
+                .replace("不少于", "")
                 .replace("小于", "")
                 .replace("大于", "")
                 .replace("等于", "")
                 .replace("个", "")
+                .replace("持有", "")
+                .replace("以上", "")
+                .replace("以内", "")
+                .replace("的", "")
+                .replace("(含7天)", "")
+                .replace("份额持有时间", "")
             ).split("，")
             for i in range(int(len(a) / 2))
         ]
+        # ['赎回时份额持有7天以内的', '1.50%', '持有7天以上(含7天),30天以内的', '0.10%', '赎回时份额持有满30天以上(含30天)的', '0.00%']
+        # ['由于本基金最短持有期限为三年,赎回费率设置为零。', '0.00%', '对持续持有期少于7日的投资者收取不低于1.5%的赎回费。', '1.50%']
+        #  ['对持续持有期少于7日的投资者收取1.5%的赎回费并全额计入基金财产', '1.50%', '对于持续持有期大于等于7日的投资者不收取赎回费用。', '0.00%']
+        # print(b)
         for j, tem in enumerate(b):
             for i, num in enumerate(tem):
                 if num[-1] == "天":
                     num = int(num[:-1])
                 elif num[-1] == "月":
                     num = int(num[:-1]) * 30
+                elif num == ".5年":
+                    num = 183
                 else:
-                    num = int(num[:-1]) * 365
+                    num = int(float(num[:-1]) * 365)
                 b[j][i] = num
         if len(b[0]) == 1:  # 有时赎回费会写大于等于一天
             b[0].insert(0, 0)
@@ -697,7 +875,7 @@ class fundinfo(basicinfo):
         con = rget(self._updateurl)
         soup = BeautifulSoup(con.text, "lxml")
         items = soup.findAll("td")
-        if dt.datetime.strptime(str(items[0].string), "%Y-%m-%d") == today():
+        if dt.datetime.strptime(str(items[0].string), "%Y-%m-%d") == today_obj():
             diffdays += 1
         if diffdays <= 10:
             self._updateurl = (
@@ -758,6 +936,21 @@ class fundinfo(basicinfo):
             self.price = self.price.append(df, ignore_index=True, sort=True)
             return df
 
+    def get_holdings(self, year="", season="", month="", category="stock"):
+        return get_fund_holdings(
+            self.code, year, season=season, month=month, category=category
+        )
+
+    def get_stock_holdings(self, year="", season="", month=""):
+        return get_fund_holdings(
+            self.code, year, season=season, month=month, category="stock"
+        )
+
+    def get_bond_holdings(self, year="", season="", month=""):
+        return get_fund_holdings(
+            self.code, year, season=season, month=month, category="bond"
+        )
+
 
 class indexinfo(basicinfo):
     """
@@ -779,6 +972,10 @@ class indexinfo(basicinfo):
         self, code, value_label=0, fetch=False, save=False, path="", form="csv"
     ):
         date = yesterday()
+        if code.startswith("SH") and code[2:].isdigit():
+            code = "0" + code[2:]
+        elif code.startswith("SZ") and code[2:].isdigit():
+            code = "1" + code[2:]
         self.rate = 0
         self._url = (
             "http://quotes.money.163.com/service/chddata.html?code="
@@ -910,7 +1107,9 @@ class cashinfo(basicinfo):
         self.interest = interest
         start = convert_date(start)
         self.start = start
-        super().__init__("mf", value_label=value_label)
+        super().__init__(
+            "mf", value_label=value_label, fetch=False, save=False, path="nobackend"
+        )  # 永远不缓存 cashinfo
 
     def _basic_init(self):
         self.name = "货币基金"
@@ -955,6 +1154,8 @@ class mfundinfo(basicinfo):
         path="",
         form="csv",
     ):
+        if code.startswith("M") and code[1:].isdigit():
+            code = code[1:]
         self._url = "http://fund.eastmoney.com/pingzhongdata/" + code + ".js"
         self.rate = 0
         super().__init__(
@@ -974,10 +1175,12 @@ class mfundinfo(basicinfo):
 
         l = eval(
             re.match(
-                r".*Data_millionCopiesIncome = ([^;]*);.*", self._page.text
+                r"[\s\S]*Data_millionCopiesIncome = ([^;]*);[\s\S]*", self._page.text
             ).groups()[0]
         )
-        self.name = re.match(r".*fS_name = \"([^;]*)\";.*", self._page.text).groups()[0]
+        self.name = re.match(
+            r"[\s\S]*fS_name = \"([^;]*)\";[\s\S]*", self._page.text
+        ).groups()[0]
         tz_bj = dt.timezone(dt.timedelta(hours=8))
         datel = [
             dt.datetime.fromtimestamp(int(d[0]) / 1e3, tz=tz_bj).replace(tzinfo=None)
@@ -998,6 +1201,8 @@ class mfundinfo(basicinfo):
             }
         )
         df = df[df["date"].isin(opendate)]
+        if len(df) == 0:
+            raise ParserFailure("no price table for %s" % self.code)
         df = df.reset_index(drop=True)
         self.price = df[df["date"] <= yesterdaydash()]
 
@@ -1086,7 +1291,7 @@ class mfundinfo(basicinfo):
         con = rget(self._updateurl)
         soup = BeautifulSoup(con.text, "lxml")
         items = soup.findAll("td")
-        if dt.datetime.strptime(str(items[0].string), "%Y-%m-%d") == today():
+        if dt.datetime.strptime(str(items[0].string), "%Y-%m-%d") == today_obj():
             diffdays += 1
         if diffdays <= 10:
             # caution: there may be today data!! then a day gap will be in table
@@ -1150,3 +1355,9 @@ class mfundinfo(basicinfo):
         if len(df) != 0:
             self.price = self.price.append(df, ignore_index=True, sort=True)
             return df
+
+
+FundInfo = fundinfo
+MFundInfo = mfundinfo
+CashInfo = cashinfo
+IndexInfo = indexinfo
