@@ -7,7 +7,7 @@ import pandas as pd
 from pyecharts import options as opts
 from pyecharts.charts import Pie, ThemeRiver
 
-from xalpha.cons import convert_date, myround, pie_opts, yesterdaydash, yesterdayobj
+from xalpha.cons import convert_date, myround, yesterdaydash, yesterdayobj
 from xalpha.evaluate import evaluate
 from xalpha.exceptions import FundTypeError, TradeBehaviorError
 from xalpha.record import record, irecord
@@ -58,12 +58,16 @@ class mul:
                     self.is_in = True
                 break
         else:
+            fundtradeobj = []
             # warning: not a very good way to automatic generate these fund obj
             # because there might be some funds use round_down for share calculation, ie, label=2 must be given
             # unless you are sure corresponding funds are added to the droplist
-            fundtradeobj = []
+        fundcodelist = [f.code for f in fundtradeobj]
+        if status is not None:
             for code in status.columns[1:]:
                 # r1, d2, v4 p = r+d+v
+                if code in fundcodelist:
+                    continue
                 p = property.get(code, 0)
                 round_label = p % 2
                 dividend_label = ((p - round_label) / 2) % 2
@@ -98,12 +102,13 @@ class mul:
                             status,
                         )
                     )
-            if istatus:
+            if istatus is not None:
                 self.is_in = True
                 if isinstance(istatus, irecord):
                     istatus = istatus.status
                 for code in istatus.code.unique():
-                    fundtradeobj.append(itrade(code, istatus))
+                    if code not in fundcodelist and not code.startswith("#"):
+                        fundtradeobj.append(itrade(code, istatus))
         self.fundtradeobj = tuple(fundtradeobj)
         self.totcftable = self._mergecftb()
 
@@ -187,6 +192,8 @@ class mul:
 
         return summarydf[columns].sort_values(by="基金现值", ascending=False)
 
+    summary = combsummary
+
     def _mergecftb(self):
         """
         merge the different cftable for different funds into one table
@@ -228,7 +235,7 @@ class mul:
         )
         return case
 
-    def v_positions(self, date=yesterdayobj(), vopts=None):
+    def v_positions(self, date=yesterdayobj(), rendered=True):
         """
         pie chart visualization of positions ratio in combination
         """
@@ -240,15 +247,27 @@ class mul:
             key=lambda x: x[1],
             reverse=True,
         )
-
         pie = Pie()
-        if vopts is None:
-            vopts = pie_opts
-        pie.add(series_name="总值占比", data_pair=sdata)
-        pie.set_global_opts(**vopts)
-        return pie.render_notebook()
+        pie.add(
+            series_name="总值占比",
+            data_pair=sdata,
+            label_opts=opts.LabelOpts(is_show=False, position="center"),
+        ).set_global_opts(
+            legend_opts=opts.LegendOpts(
+                pos_left="left", type_="scroll", orient="vertical"
+            )
+        ).set_series_opts(
+            tooltip_opts=opts.TooltipOpts(
+                trigger="item", formatter="{a} <br/>{b}: {c} ({d}%)"
+            ),
+        )
 
-    def v_positions_history(self, end=yesterdaydash(), **vkwds):
+        if rendered:
+            return pie.render_notebook()
+        else:
+            return pie
+
+    def v_positions_history(self, end=yesterdaydash(), rendered=True):
         """
         river chart visulization of positions ratio history
         use text size to avoid legend overlap in some sense, eg. legend_text_size=8
@@ -274,10 +293,12 @@ class mul:
             label_opts=opts.LabelOpts(is_show=False),
             singleaxis_opts=opts.SingleAxisOpts(type_="time", pos_bottom="10%"),
         )
+        if rendered:
+            return tr.render_notebook()
+        else:
+            return tr
 
-        return tr.render_notebook()
-
-    def v_tradevolume(self, freq="D"):
+    def v_tradevolume(self, freq="D", rendered=True):
         """
         visualization on trade summary of the funds combination
 
@@ -285,7 +306,7 @@ class mul:
             W for week and M for month, namely the trade volume is shown based on the time unit
         :returns: ``pyecharts.Bar()``
         """
-        return vtradevolume(self.totcftable, freq=freq)
+        return vtradevolume(self.totcftable, freq=freq, rendered=rendered)
 
 
 class mulfix(mul, indicator):
@@ -310,6 +331,7 @@ class mulfix(mul, indicator):
         self,
         *fundtradeobj,
         status=None,
+        istatus=None,
         property=None,
         fetch=False,
         save=False,
@@ -321,6 +343,7 @@ class mulfix(mul, indicator):
         super().__init__(
             *fundtradeobj,
             status=status,
+            istatus=istatus,
             property=property,
             fetch=fetch,
             save=save,
@@ -333,6 +356,7 @@ class mulfix(mul, indicator):
         nst = mulfix._vcash(totmoney, self.totcftable, cashobj)
         cashtrade = trade(cashobj, nst)
         # 		 super().__init__(*self.fundtradeobj, cashtrade)
+        self.cashobj = cashobj
         self.fundtradeobj = list(self.fundtradeobj)
         self.fundtradeobj.append(cashtrade)
         self.fundtradeobj = tuple(self.fundtradeobj)
@@ -343,6 +367,7 @@ class mulfix(mul, indicator):
             data={"date": [nst.iloc[0].date], "cash": [-totmoney]}
         )
 
+    @staticmethod
     def _vcash(totmoney, totcftable, cashobj):
         """
         return a virtue status table with a mf(cash) column based on the given tot money and cftable
@@ -376,19 +401,30 @@ class mulfix(mul, indicator):
 
 
 class imul(mul):
-    def __init__(self, *fundtradeobj, status=None):
+    def __init__(self, *fundtradeobj, status=None, istatus=None):
         """
         对场内投资组合进行分析的类
 
         :param fundtradeobj: itrade objects.
         :param status: 场内格式记账单，或 irecord 对象。
         """
-        if isinstance(status, irecord):
-            status = status.status
+
         if not fundtradeobj:
             fundtradeobj = []
+        if status is None:
+            status = istatus
+        if isinstance(status, irecord):
+            status = status.status
+        fundcodelist = [f.code for f in fundtradeobj]
+        if status is not None:
             for code in status.code.unique():
-                fundtradeobj.append(itrade(code, status))
+                if code not in fundcodelist and not code.startswith("#"):
+                    fundtradeobj.append(itrade(code, status))
         self.fundtradeobj = tuple(fundtradeobj)
         self.totcftable = self._mergecftb()
         self.is_in = True
+
+
+Mul = mul
+MulFix = mulfix
+IMul = imul
