@@ -3,16 +3,20 @@
 module for trade class
 """
 import datetime as dt
+import logging
 
 import pandas as pd
 from pyecharts.charts import Bar, Line
-from pyecharts.options import AxisOpts, DataZoomOpts
+from pyecharts import options as opts
 
 import xalpha.remain as rm
 from xalpha.cons import convert_date, line_opts, myround, xirr, yesterdayobj
 from xalpha.exceptions import ParserFailure, TradeBehaviorError
 from xalpha.record import irecord
-from xalpha.universal import get_daily, get_rt
+import xalpha.universal as xu
+from xalpha.universal import get_rt
+
+logger = logging.getLogger(__name__)
 
 
 def xirrcal(cftable, trades, date, guess=0.1):
@@ -42,7 +46,7 @@ def xirrcal(cftable, trades, date, guess=0.1):
                 fund.remtable[fund.remtable["date"] <= date].iloc[-1].rem,
             )[1]
         else:  # 场内交易
-            pricedf = get_daily(fund.code, end=date.strftime("%Y%m%d"), prev=20)
+            pricedf = xu.get_daily(fund.code, end=date.strftime("%Y%m%d"), prev=20)
             price = pricedf[pricedf.date <= date].iloc[-1]["close"]
             rede += fund.cftable.share.sum() * price
     cashflow.append((date, rede))
@@ -80,7 +84,7 @@ def turnoverrate(cftable, end=yesterdayobj()):
     return turnover * 365 / (end - start).days
 
 
-def vtradevolume(cftable, freq="D"):
+def vtradevolume(cftable, freq="D", rendered=True):
     """
     aid function on visualization of trade summary
 
@@ -142,6 +146,8 @@ def vtradevolume(cftable, freq="D"):
     else:
         raise ParserFailure("no such freq tag supporting")
 
+    buydata = [[d, round(x, 1)] for d, x in buydata]
+    selldata = [[d, round(x, 1)] for d, x in selldata]
     bar = Bar()
     bar.add_xaxis(datedata)
     # buydata should before selldata, since emptylist in the first line would make the output fig empty: may be bug in pyecharts
@@ -149,11 +155,13 @@ def vtradevolume(cftable, freq="D"):
     bar.add_yaxis(series_name="卖出", yaxis_data=selldata, category_gap="90%")
 
     bar.set_global_opts(
-        xaxis_opts=AxisOpts(type_="time"),
-        datazoom_opts=[DataZoomOpts(range_start=99, range_end=100)],
+        xaxis_opts=opts.AxisOpts(type_="time"),
+        datazoom_opts=[opts.DataZoomOpts(range_start=99, range_end=100)],
     )
-
-    return bar.render_notebook()
+    if rendered:
+        return bar.render_notebook()
+    else:
+        return bar
 
 
 class trade:
@@ -175,6 +183,7 @@ class trade:
         code = self.aim.code
         self.code = code
         self.name = self.aim.name
+        self.price = self.aim.price
         self.cftable = pd.DataFrame([], columns=["date", "cash", "share"])
         self.remtable = pd.DataFrame([], columns=["date", "rem"])
         self.status = status.loc[:, ["date", code]]
@@ -230,13 +239,14 @@ class trade:
                 lastdate += pd.Timedelta(1, unit="d")
                 if (lastdate - yesterdayobj()).days >= 1:
                     raise Exception("no other info to be add into cashflow table")
+            if (lastdate - yesterdayobj()).days >= 1:
+                raise Exception("no other info to be add into cashflow table")
             date = lastdate
             label = self.aim.dividend_label  # 现金分红 0, 红利再投 1
             cash = 0
             share = 0
             rem = self.remtable.iloc[-1].rem
             rdate = date
-
             if (date in recorddate) and (date not in self.aim.zhesuandate):
                 # deal with buy and sell and label the fenhongzaitouru, namely one label a 0.05 in the original table to label fenhongzaitouru
                 value = self.status[self.status["date"] == date].iloc[0].loc[code]
@@ -271,11 +281,7 @@ class trade:
                 cash += dcash
                 share += dshare
             if date in self.aim.specialdate:  # deal with fenhong and xiazhe
-                comment = (
-                    self.aim.price[self.aim.price["date"] == date]
-                    .iloc[0]
-                    .loc["comment"]
-                )
+                comment = self.price[self.price["date"] == date].iloc[0].loc["comment"]
                 if isinstance(comment, float):
                     if comment < 0:
                         dcash2, dshare2 = (
@@ -298,7 +304,7 @@ class trade:
                                 sum(self.cftable.loc[:, "share"])
                                 * (
                                     comment
-                                    / self.aim.price[self.aim.price["date"] == date]
+                                    / self.price[self.price["date"] == date]
                                     .iloc[0]
                                     .netvalue
                                 )
@@ -309,7 +315,7 @@ class trade:
                     cash += dcash2
                     share += dshare2
                 else:
-                    raise ParserFailure("comments not recoginized")
+                    raise ParserFailure("comments not recognized")
 
         self.cftable = self.cftable.append(
             pd.DataFrame([[rdate, cash, share]], columns=["date", "cash", "share"]),
@@ -387,7 +393,7 @@ class trade:
         return df
 
     def get_netvalue(self, date=yesterdayobj()):
-        return self.aim.price[self.aim.price["date"] <= date].iloc[-1].netvalue
+        return self.price[self.price["date"] <= date].iloc[-1].netvalue
 
     def briefdailyreport(self, date=yesterdayobj()):
         """
@@ -431,27 +437,28 @@ class trade:
             unitcost = 0
         return unitcost
 
-    def v_tradevolume(self, freq="D"):
+    def v_tradevolume(self, freq="D", rendered=True):
         """
         visualization on trade summary
 
         :param freq: string, "D", "W" and "M" are supported
         :returns: pyecharts.charts.bar.render_notebook()
         """
-        return vtradevolume(self.cftable, freq=freq)
+        return vtradevolume(self.cftable, freq=freq, rendered=rendered)
 
-    def v_tradecost(self, start=None, end=yesterdayobj(), vopts=None):
+    def v_tradecost(self, start=None, end=yesterdayobj(), rendered=True):
         """
         visualization giving the average cost line together with netvalue line
 
-        :param vopts: global option for line in pyecharts
         :returns: pyecharts.line
         """
         funddata = []
         costdata = []
-        pprice = self.aim.price[self.aim.price["date"] <= end]
+        pprice = self.price[self.price["date"] <= end]
+        pcftable = self.cftable
         if start is not None:
             pprice = pprice[pprice["date"] >= start]
+            pcftable = pcftable[pcftable["date"] >= start]
         for _, row in pprice.iterrows():
             date = row["date"]
             funddata.append(row["netvalue"])
@@ -460,21 +467,78 @@ class trade:
                 cost = self.unitcost(date)
             costdata.append(cost)
 
+        coords = []
+        for i, r in pcftable.iterrows():
+            coords.append(
+                [r.date, pprice[pprice["date"] <= r.date].iloc[-1]["netvalue"]]
+            )
+
+        upper = self.cftable.cash.abs().max()
+        lower = self.cftable.cash.abs().min()
+        if upper == lower:
+            upper = 2 * lower
+
+        def marker_factory(x, y):
+            buy = self.cftable[self.cftable["date"] <= x].iloc[-1]["cash"]
+            if buy < 0:
+                color = "#ff7733"
+            else:
+
+                color = "#3366ff"
+            size = (abs(buy) - lower) / (upper - lower) * 5 + 5
+            return opts.MarkPointItem(
+                coord=[x.date(), y],
+                itemstyle_opts=opts.ItemStyleOpts(color=color),
+                # this nested itemstyle_opts within MarkPointItem is only supported for pyechart>1.7.1
+                symbol="circle",
+                symbol_size=size,
+            )
+
         line = Line()
-        if vopts is None:
-            vopts = line_opts
 
         line.add_xaxis([d.date() for d in pprice.date])
-        line.add_yaxis(series_name="基金净值", y_axis=funddata, is_symbol_show=False)
-        line.add_yaxis(series_name="持仓成本", y_axis=costdata, is_symbol_show=False)
-        line.set_global_opts(**vopts)
-        return line.render_notebook()
+        line.add_yaxis(
+            series_name="基金净值", y_axis=funddata, is_symbol_show=False,
+        )
+        line.add_yaxis(
+            series_name="持仓成本",
+            y_axis=costdata,
+            is_symbol_show=False,
+            markpoint_opts=opts.MarkPointOpts(
+                data=[marker_factory(*c) for c in coords],
+            ),
+        )
+        line.set_global_opts(
+            datazoom_opts=[
+                opts.DataZoomOpts(
+                    is_show=True, type_="slider", range_start=50, range_end=100
+                ),
+                opts.DataZoomOpts(
+                    is_show=True,
+                    type_="slider",
+                    orient="vertical",
+                    range_start=50,
+                    range_end=100,
+                ),
+            ],
+            tooltip_opts=opts.TooltipOpts(
+                is_show=True,
+                trigger="axis",
+                trigger_on="mousemove",
+                axis_pointer_type="cross",
+            ),
+        )
+        if rendered:
+            return line.render_notebook()
+        else:
+            return line
 
-    def v_totvalue(self, end=yesterdayobj(), vopts=None):
+    def v_totvalue(self, end=yesterdayobj(), rendered=True, vopts=None):
         """
         visualization on the total values daily change of the aim
         """
-        partp = self.aim.price[self.aim.price["date"] >= self.cftable.iloc[0].date]
+        partp = self.price[self.price["date"] >= self.cftable.iloc[0].date]
+        # 多基金账单时起点可能非该基金持有起点
         partp = partp[partp["date"] <= end]
 
         date = [d.date() for d in partp.date]
@@ -489,11 +553,13 @@ class trade:
         line.add_xaxis(date)
         line.add_yaxis(series_name="持仓总值", y_axis=valuedata, is_symbol_show=False)
         line.set_global_opts(**vopts)
-
-        return line.render_notebook()
+        if rendered:
+            return line.render_notebook()
+        else:
+            return line
 
     def __repr__(self):
-        return self.aim.name + " 交易情况"
+        return self.name + " 交易情况"
 
 
 """
@@ -524,6 +590,16 @@ class itrade(trade):
         else:
             self.status = status[status.code == code]
         # self.cftable = pd.DataFrame([], columns=["date", "cash", "share"])
+        try:
+            self.price = xu.get_daily(
+                self.code, start=self.status.iloc[0]["date"].strftime("%Y-%m-%d")
+            )
+            self.price["netvalue"] = self.price["close"]
+        except Exception as e:
+            logger.warning(
+                "%s when trade trying to get daily price of %s" % (e, self.code)
+            )
+            self.price = None
         self._arrange()
         if not name:
             try:
@@ -535,15 +611,26 @@ class itrade(trade):
         d = {"date": [], "cash": [], "share": []}
         for _, r in self.status.iterrows():
             d["date"].append(r.date)
-            d["cash"].append(-r.value * r.share - r.fee)  # 手续费总是正的，和买入同号
-            d["share"].append(r.share)
+            if r.share == 0:
+                d["cash"].append(-r.value)
+                d["share"].append(0)
+            elif r.value == 0:
+                d["cash"].append(0)
+                d["share"].append(r.share)  # 直接记录总的应增加+或减少的份额数
+            else:
+                d["cash"].append(-r.value * r.share - abs(r.fee))  # 手续费总是正的，和买入同号
+                d["share"].append(r.share)
         self.cftable = pd.DataFrame(d)
 
-    def v_totvalue(self, end=yesterdayobj(), vopts=None):
-        raise NotImplementedError()
-
-    def v_tradecost(self, start=None, end=yesterdayobj(), vopts=None):
-        raise NotImplementedError()
-
     def get_netvalue(self, date=yesterdayobj()):
-        return get_daily(self.code, end=date.strftime("%Y%m%d"), prev=20).iloc[-1].close
+        if self.price is None:
+            return 0
+        df = self.price[self.price["date"] <= date]
+        if len(df) > 0:
+            return df.iloc[-1].close
+        else:
+            return 0
+
+
+Trade = trade
+ITrade = itrade
